"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const tf = __importStar(require("@tensorflow/tfjs-node-gpu"));
const hanzi = require('hanzi-tools');
function charsToInput(chars, charSetArray, maxParagraphLen) {
    const input = new Array(maxParagraphLen - 1).fill(0);
    let needTraditionalize = false;
    const newChars = Object.assign([], chars);
    for (let i = 0; i < chars.length; i++) {
        let charSetIdx = charSetArray.indexOf(chars[i]);
        if (charSetIdx < 0) {
            charSetIdx = charSetArray.indexOf(hanzi.traditionalize(chars[i]));
            if (charSetIdx < 0) {
                throw new Error(`Unkown character: ${chars[i]}`);
            }
            else {
                needTraditionalize = true;
            }
        }
        if (needTraditionalize) {
            newChars[i] = hanzi.traditionalize(chars[i]);
        }
        input[input.length - chars.length + i] = charSetArray.indexOf(chars[i]);
    }
    return input;
}
function oneHotToChar(onehot, charSetArray) {
    let maxIdx = 0;
    let maxValue = onehot[0];
    for (let i = 0; i < onehot.length; i++) {
        if (onehot[i] > maxValue) {
            maxValue = onehot[i];
            maxIdx = i;
        }
    }
    return charSetArray[maxIdx];
}
const lstmModel = (data, args) => __awaiter(void 0, void 0, void 0, function* () {
    const { recoverPath, loss = 'categoricalCrossentropy', optimizer = 'adam', metrics = ['accuracy'] } = args;
    let { labelMap, maxLineLength } = (args === null || args === void 0 ? void 0 : args.dataset) || {};
    if (!recoverPath) {
        labelMap = data.metadata.labelMap;
        maxLineLength = data.metadata.maxLineLength;
    }
    console.info(`charset length is ${labelMap.length} and maxline length is ${maxLineLength}`);
    let model = null;
    if (recoverPath) {
        model = (yield tf.loadLayersModel('file://' + path_1.join(recoverPath, 'model.json')));
    }
    else {
        const localModel = tf.sequential();
        localModel.add(tf.layers.embedding({
            inputDim: labelMap.length,
            outputDim: 100,
            inputLength: maxLineLength - 1
        }));
        localModel.add(tf.layers.bidirectional({
            layer: tf.layers.lstm({
                units: 150,
                returnSequences: true
            })
        }));
        localModel.add(tf.layers.dropout({
            rate: 0.2
        }));
        localModel.add(tf.layers.lstm({
            units: 100
        }));
        localModel.add(tf.layers.dense({
            units: Math.ceil(labelMap.length / 2),
            activation: 'relu',
            kernelRegularizer: tf.regularizers.l2({ l2: 0.01 })
        }));
        localModel.add(tf.layers.dense({
            units: labelMap.length,
            activation: 'softmax'
        }));
        model = localModel;
    }
    model.compile({ loss, optimizer, metrics });
    model.summary();
    return {
        model,
        metrics,
        predict: function (inputData) {
            return __awaiter(this, void 0, void 0, function* () {
                let chars = inputData.data || '';
                const sentenceLength = 4;
                for (let j = 0; j < sentenceLength; j++) {
                    for (let i = 1; i <= maxLineLength - 1; i++) {
                        const inputs = charsToInput(chars, labelMap, maxLineLength);
                        const [value] = this.model.predict(tf.tensor(inputs, [1, maxLineLength - 1])).arraySync();
                        const char = oneHotToChar(value, labelMap);
                        chars += char;
                        if (char === '。') {
                            break;
                        }
                    }
                }
                return chars;
            });
        }
    };
});
exports.default = lstmModel;
//# sourceMappingURL=index.js.map